#version 120

#define VOLUMETRIC_CLOUDS
#define DYNAMIC_AMBIENT

#define CLOUD_MODE  // Cloud mode: ON=ADD OFF=MIX
#define CLOUD_MIN 120.0 // Minimum cloud height
#define CLOUD_MAX 150.0 // Maximum cloud height

#define UNDERWATER_DARKNESS 1.5
#define LAVA_BRIGHTNESS 2.0
#define UNDERWATER_FOG_DENSITY 0.1
#define LAVA_FOG_DENSITY 0.3

const int RGBA8 = 1;
const int RGB16 = 2;
const int RGBA16 = 3;
const int gcolorFormat = RGBA16;
const int gdepthFormat = RGBA8;
const int gnormalFormat = RGB16;

const int noiseTextureResolution = 32;

#define SHADOW_MAP_BIAS 0.3

const vec3 torchcolor = vec3(1.2, 0.39, 0.02);

#define TORCH_POWER 4.0
#define TORCH_INTENSITY 2.0

varying vec4 texcoord;
varying vec4 heldLight;

varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset3;
varying vec3 sunlight_color;
varying vec3 ambient_color;

varying float transition_fading;
varying float ambient_transition;
varying float gp00;
varying float gp11;
varying float gp22;
varying float gp32;
varying float gp2232;
varying vec2 igp;
varying float flicker;
varying float timeMoon;
varying float timeLightTransition;
varying float timeSun;

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gnormal;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D depthtex0;
uniform sampler2D depthtex2;
uniform sampler2D watershadow;
uniform sampler2D noisetex;

uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform vec3 cameraPosition;

uniform float far;
uniform float near;
uniform float rainStrength;
uniform float frameTimeCounter;
uniform float viewWidth;
uniform float viewHeight;

uniform int isEyeInWater;
uniform int worldTime;

vec3 proj(in vec3 pos) {
    return vec3(pos.x * gp00, pos.y * gp11, gp2232) / pos.z + 0.5;
}

vec3 invproj(in vec3 p) {
    vec3 pos = p - 0.5;
    float z = gp32 / (pos.z + gp22);
    return vec3(pos.xy * igp, -1.0) * z;
}

float ld(in float depth) {
    return (2.0 * near) / (far + near - (2.0 * depth - 1.0) * (far - near));
}

float noise(in vec3 x) {
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f * f * (3.0 - 2.0 * f);
    vec2 uv = (p.xy + vec2(37.0, 17.0) * p.z) + f.xy;
    return mix(texture2D(noisetex, uv/noiseTextureResolution).x, 
               texture2D(noisetex, (uv + vec2(37.0, 17.0))/noiseTextureResolution).x, f.z);
}

float getCloudNoise(in vec3 worldPos) {
    vec3 coord = worldPos;
    coord.x += frameTimeCounter * 3.0;
    coord *= 0.12;
    float n = noise(coord) * 0.5;
    n += noise(coord * 3.0) * 0.35;
    n += noise(coord * 9.0) * 0.125;
    n += noise(coord * 27.0) * 0.0625;
    
    float dayOpacity = 1.0;
    float nightOpacity = 0.0; // Set to 0.0 for a black sky at night
    float cloudOpacity = mix(nightOpacity, dayOpacity, 1.0 - timeMoon);
    
    cloudOpacity = mix(cloudOpacity, (dayOpacity + nightOpacity) * 0.5, timeLightTransition);
    
    float cloudSize = mix(0.8, 1.2, rainStrength);
    cloudOpacity = mix(cloudOpacity, 1.0, rainStrength * 0.5);
    
    return max(n * cloudSize * cloudOpacity, 0.0);
}

float cloudRayMarching(in vec3 start, in vec3 end) {
    vec3 dir = normalize(end - start);
    float ay = abs(dir.y);
    float cloudMin = mix(CLOUD_MIN, CLOUD_MIN * 0.9, rainStrength);
    float cloudMax = mix(CLOUD_MAX, CLOUD_MAX * 1.1, rainStrength);
    float cutoff = 0.00001;
    
    if (max(start.y, end.y) < cloudMin || cloudMax < min(start.y, end.y) || ay < cutoff) {
        return 0.0;
    }
    
    float sum = 0.0;
    vec3 ray = start + (max(0.0, cloudMin - start.y) - max(0.0, start.y - cloudMax)) * dir / dir.y;
    vec3 step = dir / ay;
    float wall = mix(max(cloudMin, end.y), min(cloudMax, end.y), step.y * 0.5 + 0.5);
    float dest = min(ray.y * step.y + 24.0, wall * step.y) * step.y;
    float dy = abs(dest - ray.y);
    
    for (int i = 0; i < 32; i++) {
        if (i >= int(dy)) break;
        ray += step;
        sum += getCloudNoise(ray) * 0.03;
    }
    
    float dayIntensity = 1.0;
    float nightIntensity = 0.0; // Set to 0.0 for no cloud glow at night
    float cloudIntensity = mix(nightIntensity, dayIntensity, 1.0 - timeMoon);
    cloudIntensity = mix(cloudIntensity, (dayIntensity + nightIntensity) * 0.5, timeLightTransition);
    
    return sum * min(1.0, (ay - cutoff) * 5) * cloudIntensity;
}

void main() {
    // Retrieve the base color from the gcolor texture
    vec3 color = texture2D(gcolor, texcoord.st).rgb;
    float spec = 0.0;

    // Get auxiliary information from gaux1 texture
    vec4 aux = texture2D(gaux1, texcoord.st);
    
    // Calculate eye position and normal in world space
    vec3 eye = invproj(vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r));
    vec3 neye = normalize(eye);
    vec4 wpos = gbufferModelViewInverse * vec4(eye, 1.0);
    float dist = length(eye);
    vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;
    vec3 nnormal = normalize(normal);
    
    // Check if the player is not underwater
    float isEyeNotInWater = float(isEyeInWater < 0.5);
    float no_hand = float(aux.g < 0.35 || 0.45 < aux.g);

    // Handle volumetric clouds
    #ifdef VOLUMETRIC_CLOUDS
    #ifdef CLOUD_MODE
    vec3 cloudColor = mix(vec3(0.0), vec3(0.2235, 0.4078, 0.5608), timeSun);
    cloudColor = mix(cloudColor, vec3(0.0), rainStrength * (1.0 - timeSun));
    float cloudIntensity = 0.6 + 0.4 * timeSun;
    vec3 cloudContribution = cloudRayMarching(cameraPosition, cameraPosition + wpos.xyz) * 
                            cloudColor * 
                            (no_hand * 0.6 + 0.4) * 
                            cloudIntensity;
    cloudContribution *= mix(vec3(0.7, 0.7, 1.0), vec3(1.0), timeSun);
    color += cloudContribution;
    #else
    vec3 cloudColor = vec3(0.0, 0.0, 0.0); // Black sky at night
    float cloudOpacity = 0.5;
    float cloudAmount = cloudRayMarching(cameraPosition, cameraPosition + wpos.xyz) * 
                       (no_hand * 0.6 + 0.4) * 
                       cloudOpacity;
    color = mix(color, cloudColor, cloudAmount);
    #endif
    #endif

    // Fog system
    float fogStart = 0.2;
    float fogEnd = 0.8;
    float fogDensity = mix(0.0, 0.006, rainStrength);

    vec3 dayFogColor = vec3(0.8863, 0.8863, 0.9412);
    vec3 nightFogColor = vec3(0.0, 0.0, 0.0); // Black fog at night
    vec3 sunsetFogColor = vec3(0.8392, 0.6784, 0.6157);

    vec3 finalFogColor;
    if (timeLightTransition > 0.0) {
        finalFogColor = mix(
            mix(dayFogColor, nightFogColor, timeMoon),
            sunsetFogColor,
            timeLightTransition
        );
    } else {
        finalFogColor = mix(nightFogColor, dayFogColor, timeSun);
    }

    finalFogColor = mix(finalFogColor, vec3(0.0), rainStrength * (1.0 - timeSun));

    float fogFactor = clamp((dist - fogStart) / (fogEnd - fogStart), 0.0, 1.0);
    fogFactor = 1.0 - exp(-fogDensity * dist);
    color = mix(color, finalFogColor, fogFactor * isEyeNotInWater);

    // Underwater effects
    if (isEyeInWater > 0.5) {
        vec3 waterColor = vec3(0.2, 0.4, 0.6); // Light blue underwater fog
        float waterDarkness = UNDERWATER_DARKNESS;
        float waterFogDensity = UNDERWATER_FOG_DENSITY;

        color *= 1.0 - (waterDarkness * (1.0 - exp(-dist * waterFogDensity)));
        color = mix(color, waterColor, 1.0 - exp(-dist * waterFogDensity));

        vec2 causticUV = texcoord.st + vec2(
            sin(frameTimeCounter * 0.5 + texcoord.s * 10.0) * 0.01,
            cos(frameTimeCounter * 0.5 + texcoord.t * 10.0) * 0.01
        );
        float caustic = noise(vec3(causticUV * 20.0, frameTimeCounter));
        color += vec3(0.0, 0.0824, 0.1216) * caustic * (1.0 - fogFactor);
    }

    // Adjust color based on time of day
    vec3 dayColor = vec3(1.05, 1.05, 1.0);
    vec3 nightColor = vec3(0.0);
    vec3 timeAdjustment = mix(nightColor, dayColor, timeSun);
    color *= timeAdjustment;
    color *= mix(vec3(1.0), vec3(0.5, 0.5, 0.6), rainStrength * (1.0 - timeSun));

    // Add subtle color variations based on height
    float height = wpos.y;
    vec3 heightAdjustment = mix(
        vec3(1.0),
        vec3(1.05, 1.02, 0.98),
        clamp((height - 64.0) / 64.0, 0.0, 1.0)
    );
    color *= heightAdjustment;

    // Terrain enhancements
    if (0.15 < aux.g) {
        float ao = aux.r;
        color *= mix(1.0, 0.8 + 0.2 * ao, 0.8);

        vec3 worldPos = wpos.xyz + cameraPosition;
        float terrainNoise = noise(worldPos * 0.5) * 0.05;
        color *= (1.0 + terrainNoise);
    }

    // Add held light effects
    vec3 heldLightColor = heldLight.rgb * TORCH_INTENSITY;
    float heldLightFalloff = 1.0 / (1.0 + dist * dist * 0.01);
    color += heldLightColor * heldLightFalloff * heldLight.a;

    // Clamp color to valid range
    color = clamp(color, 0.0, 1.0);

    // Optional vignette effect
    vec2 screenPos = gl_FragCoord.xy / vec2(viewWidth, viewHeight);
    float vignette = 1.0 - length(screenPos - 0.5) * 0.5;
    color *= mix(1.0, vignette, 0.3);

    // Final output
    gl_FragData[0] = vec4(color, spec);
}
