#version 120

uniform mat4 gbufferProjection;
uniform mat4 gbufferModelView;

uniform vec3 sunPosition;
uniform vec3 moonPosition;

uniform float rainStrength;
uniform float frameTimeCounter;

uniform int worldTime;
uniform int heldItemId;

varying vec4 texcoord;
varying vec4 heldLight;

varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset3;
varying vec3 sunlight_color;
varying vec3 ambient_color;

varying float transition_fading;
varying float ambient_transition;
varying float gp00;
varying float gp11;
varying float gp22;
varying float gp32;
varying float gp2232;
varying vec2 igp;
varying float flicker;
varying float timeMoon;
varying float timeLightTransition;
varying float timeSun;

void main() {
    gl_Position = ftransform();
    texcoord = gl_MultiTexCoord0;

    float wtime = float(worldTime);
    
    // Improved transition timings for smoother day/night cycle
    transition_fading = 1.0 - (
        clamp((wtime - 12000.0) / 500.0, 0.0, 1.0) - 
        clamp((wtime - 13000.0) / 500.0, 0.0, 1.0) + 
        clamp((wtime - 22500.0) / 500.0, 0.0, 1.0) - 
        clamp((wtime - 23500.0) / 500.0, 0.0, 1.0)
    );
    
    ambient_transition = 1.0 - (
        clamp((wtime - 12500.0) / 300.0, 0.0, 1.0) - 
        clamp((wtime - 12800.0) / 300.0, 0.0, 1.0) + 
        clamp((wtime - 22900.0) / 300.0, 0.0, 1.0) - 
        clamp((wtime - 23200.0) / 300.0, 0.0, 1.0)
    );
    
    float isDay = float(worldTime < 12800 || 23200 < worldTime);
    lightVector = normalize(mix(moonPosition, sunPosition, isDay));

    // Enhanced sun and moon light radius
    float sunlight_radius = mix(0.08, 0.12, isDay) * (1.0 - transition_fading);
    lightOffset0 = normalize(lightVector + (gbufferModelView * vec4(0.0, sunlight_radius, 0.0, 0.0)).xyz);

    float moonlight_radius = mix(0.12, 0.08, isDay) * (1.0 - transition_fading);
    lightOffset3 = normalize(-lightVector + (gbufferModelView * vec4(0.0, moonlight_radius, 0.0, 0.0)).xyz);
    
    // Improved torch flicker effect
    flicker = sin(frameTimeCounter * 4.0) * 0.03 + 0.97;
    float heldFlicker = cos(frameTimeCounter * 3.0) * 0.05 + 0.95;

    // Enhanced held light colors
    heldLight = vec4(0.0);
    if (heldItemId == 50) heldLight = vec4(1.0, 0.7, 0.3, heldFlicker);     // torch
    if (heldItemId == 89) heldLight = vec4(1.0, 0.9, 0.5, 0.6);             // glowstone
    if (heldItemId == 91) heldLight = vec4(0.9, 0.6, 0.2, 0.7 * heldFlicker); // jack-o-lantern
    if (heldItemId == 327) heldLight = vec4(1.0, 0.5, 0.2, 0.4 * heldFlicker); // lava bucket
    heldLight += vec4(-0.05);    // subtle self-glow

    float hour = wtime / 1000.0 + 6.0;
    if (hour > 24.0) hour -= 24.0;
    
    timeMoon = clamp((hour - 12.5) / 11.0, 0.0, 1.0);
    timeLightTransition = max(0.0, 1.0 - abs(hour - 6.0) / 1.5) + max(0.0, 1.0 - abs(hour - 18.0) / 1.5);
    timeSun = 1.0 - timeMoon;
    
    // Enhanced time-based colors
    const vec3 sunlightColor[25] = vec3[25](
        vec3(10, 20, 40),  vec3(10, 20, 40),  vec3(10, 20, 40),  vec3(10, 20, 40),  vec3(15, 25, 45),
        vec3(60, 70, 90),  vec3(255, 200, 80), vec3(255, 205, 90), vec3(255, 210, 105), vec3(255, 220, 120),
        vec3(255, 225, 135), vec3(255, 235, 150), vec3(255, 245, 160), vec3(255, 235, 150), vec3(255, 225, 135),
        vec3(255, 220, 120), vec3(255, 210, 105), vec3(255, 205, 90), vec3(255, 200, 80), vec3(87, 77, 204),
        vec3(30, 50, 90),  vec3(20, 40, 80),  vec3(20, 40, 80),  vec3(20, 40, 80),  vec3(20, 40, 80)
    );

    int index = int(floor(hour));
    sunlight_color = mix(sunlightColor[index], sunlightColor[(index + 1) % 25], fract(hour)) / 255.0;
    
    // Enhanced ambient colors
    const vec3 ambientColor[25] = vec3[25](
        vec3(30, 45, 75), vec3(30, 45, 75), vec3(30, 45, 75), vec3(35, 50, 80), vec3(35, 50, 80),
       vec3(0, 0, 0),  vec3(0, 0, 0),  vec3(0, 0, 0),  vec3(0, 0, 0),  vec3(5, 10, 20),
        vec3(200, 215, 280), vec3(215, 240, 290), vec3(230, 255, 300), vec3(215, 240, 290), vec3(200, 215, 280),
        vec3(175, 200, 270), vec3(160, 186, 260), vec3(150, 170, 255), vec3(138, 160, 255), vec3(87, 77, 204),
        vec3(70, 100, 160), vec3(70, 100, 160), vec3(60, 90, 150), vec3(60, 90, 150), vec3(60, 90, 150)
    );

    ambient_color = mix(ambientColor[index], ambientColor[(index + 1) % 25], fract(hour)) / 255.0;
    
    // Projection matrix calculations
    gp00 = gbufferProjection[0][0] * -0.5;
    gp11 = gbufferProjection[1][1] * -0.5;
    gp22 = gbufferProjection[2][2] * 0.5;
    gp32 = gbufferProjection[3][2] * 0.5;
    gp2232 = -gp22 - gp32;
    igp = vec2(-1.0) / vec2(gp00, gp11);
}